package backend;

import intermediate.frame.*;
import java.util.*;
import intermediate.Temp;

/**
 * Ein Aktivierungssegment fr den Interpreter
 */
public class InterpreterFrame extends Frame {

    private Temp fp = new Temp("fp");
    private int numberOfFields, numberOfConsts;
    private int level;
    private ArrayList fields;

    /**
     * Leerer Konstruktor. Frames, die auf diese Art erzeugt werden, knnen nicht im ausfhrbaren
     * Programm verwendet werden. Sie dienen dazu, um im Zwischencode auf Werte wie wordSize() oder
     * das Temp fp zugreifen zu knnen.
     */
    public InterpreterFrame() {
    }

    /**
     * Konstruktor
     * @param numberOfFields Anzahl der Felder: reservierte, Konstanten und Variablen
     * @param numberOfConsts Anzahl der Konstanten
     * @param consts Werte der Konstanten
     */
    public InterpreterFrame(int numberOfFields, int numberOfConsts, List consts, int level) {
        this.numberOfFields = numberOfFields;
        this.numberOfConsts = numberOfConsts;
        this.level = level;
        fields = new ArrayList(numberOfFields + getMinOffset());
        for (int i = 0; i < numberOfFields + getMinOffset(); i++) {
            fields.add(i, new Integer(0));
        }
        for (int i = 0; i < numberOfConsts; i++) {
            fields.set(i + getMinOffset(), consts.get(i));
        }
    }

    /**
     * Konstruktor
     * @param numberOfFields Anzahl der Felder: reservierte, Konstanten und Variablen
     * @param numberOfConsts Anzahl der Konstanten
     * @param consts Werte der Konstanten
     */
/*    public InterpreterFrame(int numberOfFields, int numberOfConsts, int[] consts) {
        this.numberOfFields = numberOfFields;
        this.numberOfConsts = numberOfConsts;
        fields = new ArrayList(numberOfFields);
        for (int i = 0; i < numberOfConsts; i++) {
            fields.add(i + getMinOffset(), new Integer(consts[i]));
        }
    }
*/
    public int wordSize() {
        return 1;
    }

    public Temp fp() {
        return fp;
    }

    public int level() {
        return level;
    }

    /**
     * Anzahl der Felder (Variablen und Konstant, OHNE die reservierten Felder!)
     * @return
     */
    public int getNumberOfFields() {
        return numberOfFields;
    }

    public int getNumberOfConsts() {
        return numberOfConsts;
    }

    /*public int[] getFields() {
        return fields;
    }*/
    public ArrayList getFields() {
        return fields;
    }

    public int getField(int i) {
        return ((Integer)fields.get(i)).intValue();
    }

    public void setField(int i, int value) {
        fields.set(i, new Integer(value));
    }



    /**
     * Erzeugt eine 1:1 Kopie des aktuellen Frames, da bei Rekursion immer eine neue Instanz verwendet
     * werden muss. Eventuell belegte Variablenwerte werden mit kopiert, dies lsst sich verhindern, indem
     * man nur die Vorlage aus dem FrameContainer zum Klonen nutzt. Dementsprechend sollte das Frame aus dem
     * FrameContainer nicht verndert werden, sondern nur die aus ihm erzeugten Kopien.
     */
    public InterpreterFrame createClone() {
        InterpreterFrame iframe = new InterpreterFrame(numberOfFields, numberOfConsts, (List)null, level);
        ArrayList newList = new ArrayList();
        for (Iterator it = fields.iterator(); it.hasNext();) {
            newList.add(it.next());
        }
        iframe.fields = newList;
        return iframe;
    }

    public int getMinOffset() {
        return 4;
    };

    public static final int RETURN_CODEBLOCK = 3;

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("[");
        for (int i = 0; i < getFields().size(); i++) {
            sb.append((i == 0 ? "" : ",") + getFields().get(i));
        }
        sb.append("]");
        return sb.toString();
    }

}
