package frontend.parsetree;

import intermediate.*;
import java.util.*;
import intermediate.operators.*;
import antlr.Token;
import frontend.symbol.*;

public class ProcedureNode extends PL0Node {

    private SymbolTable table = SymbolTable.getSymbolTable();
    private static CodeContainer cc;
    private static FrameContainer fc;

    public ProcedureNode() {
        super();
    }

    public ProcedureNode(Token tok) {
        super(tok);
    }

    public CodeContainer createIntermediateCode() {
        cc = new CodeContainer();
        PL0Node node = (PL0Node)getFirstChild();
        createAll(node, 0);
        return cc;
    }

    /**
     * Erzeugt rekursiv die Syntaxbume fr alle Gltigkeitsbereiche. Alle erzeugten Bume
     * werden linear im sogenannten CodeContainer (HashMap) gespeichert. Dabei entspricht der Key
     * der HashMap der Reihenfolge des Auftretens des Gltigkeitsbereichs im Quellcode. Diese
     * Reihenfolge wurde in der Symboltabelle gespeichert und wird nun ausgelesen.
     */
    private void createAll(PL0Node node, int number) {
        //Prozedurnamen berspringen, der ist fr den Zwischencode irrelevant.
        //Da der erste Knoten nur in Prozeduren ein IdentNode ist, aber nicht
        //im Hauptprogramm, muss vor dem berspringen auf IdentNode getestet werden.
        if (node instanceof IdentNode) {
            node = (PL0Node)node.getNextSibling();
        }

        while (node instanceof ProcedureNode) {
            String procname = node.getFirstChild().getText();
            ScopeDef sd = (ScopeDef)table.lookup(procname);
            //Gltigkeitshierarchie nachbauen, damit die Variablen korrekt in Speicherberechnungen
            //umgewandelt werden (Folgen des Static Link)
            table.pushScope(sd);
            createAll((PL0Node)node.getFirstChild(), sd.getScopeNumber());
            table.popScope();
            node = (PL0Node)node.getNextSibling();
        }
        if (node != null) {
            cc.addCode(node.convertStatement(), number);
        }

    }


}
