package frontend.parsetree;

import intermediate.*;
import intermediate.operators.*;
import antlr.Token;

public class WhileNode extends PL0Node {

    public WhileNode(Token tok) {
        super(tok);
    }

    /**
     * Konvertiert den Knoten in ein Statement. Die Umwandlung erfolgt dabei so:
     *
     * Start:
     * while (condNode) do execNode;
     *
     * Die Knoten condNode und execNode werden in die Statements condStm und execStm umgewandelt.
     *
     * Ziel (linearisiert):
     * JUMP(l_cond)
     * LABEL(l_body)
     * execStm
     * LABEL(l_cond)
     * CJUMP(rel, par1, par2, l_body, l_end)   //rel, par1 und par2 sind der Vergleichsoperator und
     *                                         //die zu vergleichenden Ausdrcke (gewonnen aus condStm)
     * LABEL(l_end)
     */
    public Statement convertStatement() {
        Label l_cond = new Label();
        Label l_body = new Label();
        Label l_end = new Label();
        PL0Node condNode = (PL0Node)getFirstChild();
        PL0Node execNode = (PL0Node)condNode.getNextSibling();
        Statement condStm = condNode.convertConditional(l_body, l_end);
        Statement execStm = execNode.convertStatement();
        return new SEQ(new JUMP(l_cond), new SEQ(new LABEL(l_body), new SEQ(execStm,
                new SEQ(new LABEL(l_cond), new SEQ(condStm, new LABEL(l_end))))));
    };

}
